//gcc -g e3631.c -o e3631 -L/usr/local/lib/ -lusb220_gpib -lpthread \
//	`pkg-config gtk+-2.0 --cflags --libs`
// caution: You need ROOT previledge when execute this utility
#include <stdlib.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "usb220_gpib.h"

#define _USE_U2
#define MAX_USB_DEVICES				16


int fd=0;
short MyGpibAdrs = 1;	// GPIB address for USB-GPIB
char GpAdrsBuf[12];	// GPIB address for target-GPIB
char szBuf[256];
int InitFlag;

int OutPutON;			// 出力ONフラグ
int ChkBtnOutVol;		// チェックボタンの状態
char volt[8];
char amp[8];

GtkWidget *entry_amp;
GtkWidget *status_bar;
gint status_id;

static int ErrorCheck()
{
	int		GpStatus;
	char  	RcvData[256];

	// E3631A GPIBコマンド送信
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "SYST:ERR?");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "SYST:ERR?" );
#endif
	if(GpStatus != 0)
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return -1;
	}

	// GPIBバスからデータリード
	memset(RcvData, 0x00, sizeof(RcvData));
#ifdef _USE_U2
	GpStatus = U2Gpred(fd, GpAdrsBuf, RcvData, sizeof(RcvData));
#else
	GpStatus = gp_red(GpAdrsBuf, RcvData, sizeof(RcvData));
#endif
	if(GpStatus != 0)
	{
		printf("gp_red() error [%d]\n", GpStatus);
		return -2;
	}

	// E3631Aから返るコードが0であればエラー発生はなし
	if(RcvData[1] == '0' && RcvData[2] == ',')
		return 0;
	else
	{
		printf("E3631Aエラーメッセージ [%s]\n", RcvData);
		return -3;
	}
}

static void Cmd_OnCmdGpInit( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return;
	}

	// enable IFC
#ifdef _USE_U2
	GpStatus = U2Gpcli(fd);
#else
	GpStatus = gp_cli();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_cli() error [%d]\n", GpStatus);
		return;
	}

	// enable REN
#ifdef _USE_U2
	GpStatus = U2Gpren(fd);
#else
	GpStatus = gp_ren();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_ren() error [%d]\n", GpStatus);
		return;
	}

#ifdef _USE_U2
	GpStatus = U2Gpdelm(fd, "t", 0x0A);
#else
	GpStatus = gp_delm("t", 0x0A);
#endif

	// set 3 second GPIB bus timeout
#ifdef _USE_U2
	GpStatus = U2Gptmout(fd, 3);
#else
	GpStatus = gp_tmout(3);
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_tmout() error [%d]\n", GpStatus);
		return;
	}

	// Send SDC command
	// get address of GPIBdevice
#ifdef _USE_U2
	GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
	GpStatus = gp_clr( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_clr() error [%d]\n", GpStatus);
		return;
	}

	// Send LLO command
#ifdef _USE_U2
	GpStatus = U2Gpllo(fd);
#else
	GpStatus = gp_llo();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_llo() error [%d]\n", GpStatus);
		return;
	}

	// Send E3631A GPIB command
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "*RST;*CLS");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "*RST;*CLS" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "初期化正常終了");

	InitFlag = 1;
}

static void Cmd_OnCmdOFF ( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		return;
	}

	gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), " ");

	// Send E3631A GPIB command
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "OUTP OFF");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "OUTP OFF" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	GpStatus = ErrorCheck();
	if( GpStatus ) // 0以外なら終了
		return;
	else
		OutPutON = 0;
}

static void Cmd_OnCmdOK ( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;
	char  	szCmd3631[256]; // 送信データ格納用バッファ

	if(InitFlag == 0){
		return;
	}

	gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), " ");

	// 送信コマンド例 "APPL P6V, 5.0, 1.0"
	strcpy(szCmd3631, "APPL ");

	switch( ChkBtnOutVol )
	{
	case 0/*IDR_6V*/: // +6V
		strcat(szCmd3631, "P6V, ");
		break;
	case 1/*IDR_P25V*/: // +25V
		strcat(szCmd3631, "P25V, ");
		break;
	case 2/*IDR_N25V*/: // -25V
		strcat(szCmd3631, "N25V, ");
		break;
	}

	// 出力電圧値をセット
	strcat(szCmd3631, volt);

	// 出力電流値をセット
	strcat(szCmd3631, ", ");
	strcat(szCmd3631, amp);

	printf ("send command: %s\n", szCmd3631);
	// Set output parameter
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szCmd3631);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szCmd3631 );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	GpStatus = ErrorCheck();
	if( GpStatus ){ // 0以外なら終了
		printf("出力ONされていません\n", GpStatus);
		return;
	}

	// Store a state in memory location 1
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "*SAV 1");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "*SAV 1" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	GpStatus = ErrorCheck();
	if( GpStatus ){ // 0以外なら終了
		printf("出力ONされていません\n", GpStatus);
		return;
	}

	// display on
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "DISP ON");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "DISP ON" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	GpStatus = ErrorCheck();
	if( GpStatus ){ // 0以外なら終了
		return;
	}

	// Enable the outputs
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "OUTP ON");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "OUTP ON" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	GpStatus = ErrorCheck();
	if( GpStatus ) // 0以外なら終了
		return;
	else
	{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "出力中");
		OutPutON = 1;
	}
}

static void Cmd_OnCancel( GtkWidget *widget,
                      gpointer   data )
{
	int	GpStatus;

	if(InitFlag == 0){
		return;
	}

	if( OutPutON == 1 )
	{
		//Cmd_OnCmdOFF();
		// Send E3631A GPIB command
#ifdef _USE_U2
		GpStatus = U2Gpwrt(fd, GpAdrsBuf, "OUTP OFF");
#else
		GpStatus = gp_wrt( GpAdrsBuf, "OUTP OFF" );
#endif
		if ( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}

		GpStatus = ErrorCheck();
		if( GpStatus ) // 0以外なら終了
			return;
		else
			OutPutON = 0;
	}

	//send LCL command
#ifdef _USE_U2
	GpStatus = U2Gplcl(fd, GpAdrsBuf);
#else
	GpStatus = gp_lcl( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_lcl() error [%d]\n", GpStatus);
	}
}

static void enter_adrs( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  //printf ("changed contents: %s\n", entry_text);
  memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
  sprintf(GpAdrsBuf, "%s", entry_text);
}

static void enter_volt( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  memset(volt, 0x00, sizeof(volt));
  sprintf(volt, "%s", entry_text);
  //printf ("volt contents: %s\n", entry_text);
}

static void enter_amp( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  memset(amp, 0x00, sizeof(amp));
  sprintf(amp, "%s", entry_text);
  //printf ("amp contents: %s\n", entry_text);
}

static void toggle_button1( GtkWidget    *widget,
                      gpointer   data )
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
        /* もしプログラムの流れがここに到達した場合、トグル・ボタンは下がった状態です。*/
		//printf ("changed button1:\n");
		ChkBtnOutVol = 0;
		gtk_entry_set_text (GTK_ENTRY (entry_amp), "5.000");
    } else {
        /* もしプログラムの流れがここに到達した場合、トグル・ボタンは上がった状態です。*/
		//printf ("changed button2:\n");
    }
}

static void toggle_button2( GtkWidget    *widget,
                      gpointer   data )
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
		ChkBtnOutVol = 1;
		gtk_entry_set_text (GTK_ENTRY (entry_amp), "1.000");
    }
}

static void toggle_button3( GtkWidget    *widget,
                      gpointer   data )
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
		ChkBtnOutVol = 2;
		gtk_entry_set_text (GTK_ENTRY (entry_amp), "1.000");
    }
}

static void create_window( void )
{
	GtkWidget *window;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *temp_bar;
	GtkWidget *frame;
	GtkWidget *button;
	GSList *group;
	gint temp_id;
	char	buf[12];

    /* make new window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);
    gtk_widget_set_size_request (GTK_WIDGET (window), 400, 280);
    gtk_window_set_title (GTK_WINDOW (window), "e3631");
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_main_quit), NULL);

    vbox = gtk_vbox_new (FALSE, 2);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);

    /* make GPIB adrs */
    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 10);
    gtk_widget_show (hbox);

    label = gtk_label_new ("GPIB 機器アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 1);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_adrs),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "5");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
    gtk_widget_show (entry);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    label = gtk_label_new ("USB220 アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    temp_bar = gtk_statusbar_new ();
    temp_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (temp_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " %d", MyGpibAdrs);
    gtk_statusbar_push (GTK_STATUSBAR (temp_bar), temp_id, buf);
    gtk_box_pack_start (GTK_BOX (hbox), temp_bar, TRUE, TRUE, 0);
    gtk_widget_show (temp_bar);

    /* 新しいフレームを作成します。*/
    frame = gtk_frame_new ("E3631A Power Supply");
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 5);
    gtk_widget_show (frame);

    /* 縦横 6x6 のテーブルを生成します。*/
    table = gtk_table_new (6, 6, TRUE);
    gtk_container_set_border_width (GTK_CONTAINER (table), 5);
    gtk_table_set_col_spacings( GTK_TABLE (table), 2 );
    gtk_container_add (GTK_CONTAINER (frame), table);
    gtk_widget_show (table);

    button = gtk_radio_button_new_with_label (NULL, "+6V");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
    g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (toggle_button1),
                      NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 0, 2);
    gtk_widget_show (button);

    group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label (group, "+25V");
    g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (toggle_button2),
                      NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 0, 2);
    gtk_widget_show (button);

    group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label (group, "-25V");
    g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (toggle_button3),
                      NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 3, 4, 0, 2);
    gtk_widget_show (button);

    /* make setting param */
    label = gtk_label_new ("出力電圧(V)");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 3, 2, 4);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 5);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_volt),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "0");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_table_attach_defaults (GTK_TABLE (table), entry, 3, 4, 2, 4);
    gtk_widget_show (entry);

    label = gtk_label_new ("出力電流(A)");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 3, 4, 6);
    gtk_widget_show (label);

    entry_amp = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry_amp), 5);
    g_signal_connect (G_OBJECT (entry_amp), "changed",
		      G_CALLBACK (enter_amp),
		      (gpointer) entry_amp);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "5.000");
    gtk_entry_set_text (GTK_ENTRY (entry_amp), buf);
    gtk_table_attach_defaults (GTK_TABLE (table), entry_amp, 3, 4, 4, 6);
    gtk_widget_show (entry_amp);

    status_bar = gtk_statusbar_new ();
    status_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (status_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " ");
    gtk_statusbar_push (GTK_STATUSBAR (status_bar), status_id, buf);
    gtk_box_pack_start (GTK_BOX (vbox), status_bar, TRUE, TRUE, 0);
    gtk_widget_show (status_bar);

    /* make botton */
    hbox = gtk_hbox_new (FALSE, 2);
    gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 10);
    gtk_widget_show (hbox);

	button = gtk_button_new_with_label ("初期化");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdGpInit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);              

	button = gtk_button_new_with_label ("出力ON");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdOK), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);

	button = gtk_button_new_with_label ("出力OFF");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdOFF), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);

	button = gtk_button_new_with_label ("終了");
	g_signal_connect (G_OBJECT (button), "clicked",
		               G_CALLBACK (Cmd_OnCancel), NULL);
	g_signal_connect (G_OBJECT (button), "clicked",
			      G_CALLBACK (gtk_main_quit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (button);
	gtk_widget_show (button);

	gtk_widget_show (window);
}

int main( int   argc,
          char *argv[] )
{
#ifdef _USE_U2
	int id, i;
	unsigned short UnitId[MAX_USB_DEVICES];
	unsigned short MaxUnit = MAX_USB_DEVICES;

	// 引数無しの場合
	if (argc != 2) {
		MaxUnit = U2GpEnumUnit(UnitId, MaxUnit);
		for(i=0;i<MaxUnit;i++){
			printf("U2GpEnumUnit%d UnitId = %02x \n", i, UnitId[i]);
		}
		//最初に見つけたUSB220をオープン
		fd=U2GpOpenUnit(UnitId[0]);
	}else{
		id = atoi (argv[1]);
		//入力されたIDをもつUSB220をオープン
		fd=U2GpOpenUnit(id);
	}
#else
	fd=gp_attach();
#endif
	if(fd<=0){
		printf("Device not opened \n");
		return 0;
    }
	InitFlag = 0;
	OutPutON = 0;
	ChkBtnOutVol = 0;
	gp_GetDeviceID(fd, 0, (char*)&MyGpibAdrs);

	gtk_init (&argc, &argv);

	create_window();

	gtk_main ();
	gp_detach(fd);

	return 0;
}
