/**********************************************************************************
 *
 *	日置電気 電力計 3332 制御プログラム
 *
 *	機能 ： 日置電器製の電力計3332で取得した電圧値、電流値、電力値
 *         の表示が行えます。
 *
 **********************************************************************************/
//gcc -g hioki3332.c -o hioki3332 -L/usr/local/lib/ -lusb220_gpib -lpthread \
//	`pkg-config gtk+-2.0 --cflags --libs`
// caution: You need ROOT previledge when execute this utility

#include <stdlib.h>
#include <math.h>
//#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "usb220_gpib.h"

#define _USE_U2
#define MAX_USB_DEVICES				16
#define X_RANGE  240 // グラフ表示最大個数は240個


int fd=0;
short MyGpibAdrs;		// GPIB address for USB-GPIB
char GpAdrsBuf[12];	// GPIB address for target-GPIB
char szBuf[256];
int InitFlag;
char RunFlag;			// タイマーフラグ
int timer = 0;
static int Count;		// データカウンタ

long  g_setpos; // データ格納ポインタ
long  g_getcount; // データ取得個数
double  g_Val[2][X_RANGE]; // グラフ描画のためのデータ格納

GtkWidget *vol_bar;
gint vol_id;
GtkWidget *cur_bar;
gint cur_id;
GtkWidget *pow_bar;
gint pow_id;
GtkWidget *count_bar;
gint count_id;
GtkWidget *status_bar;
gint status_id;
//struct GdkGC *gdkgc;


static int InitGpMachine( void )
{
	int		GpStatus;
	char	RcvBuf[256];

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return -1;
	}

	// enable IFC
#ifdef _USE_U2
	GpStatus = U2Gpcli(fd);
#else
	GpStatus = gp_cli();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_cli() error [%d]\n", GpStatus);
		return -1;
	}

	// enable REN
#ifdef _USE_U2
	GpStatus = U2Gpren(fd);
#else
	GpStatus = gp_ren();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_ren() error [%d]\n", GpStatus);
		return -1;
	}

	// デリミタ設定
#ifdef _USE_U2
	GpStatus = U2Gpdelm(fd, "t", 0x8A);
	GpStatus = U2Gpdelm(fd, "l", 0x0A);
#else
	GpStatus = gp_delm("t", 0x8A);
	GpStatus = gp_delm("l", 0x0A);
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_delm() error [%d]\n", GpStatus);
		return -1;
	}

#ifdef _USE_U2
	GpStatus = U2Gptmout(fd, 3);
#else
	GpStatus = gp_tmout(3);
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_tmout() error [%d]\n", GpStatus);
		return -1;
	}


	// Send SDC command
	// get address of GPIBdevice
#ifdef _USE_U2
	GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
	GpStatus = gp_clr( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_clr() error [%d]\n", GpStatus);
		return -1;
	}

#ifdef _USE_U2
	// コマンド"*RST"送信
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "*RST");
	// 電圧オートレンジ
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "VOLTage:AUTO");
	// 電流オートレンジ
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "CURRent:AUTO");
	// 応答速度をFAST
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "RESPonse FAST");
	// 動作の完了をここで確認しておく
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "*OPC?");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "*RST" );
	GpStatus = gp_wrt( GpAdrsBuf, "VOLTage:AUTO" );
	GpStatus = gp_wrt( GpAdrsBuf, "CURRent:AUTO" );
	GpStatus = gp_wrt( GpAdrsBuf, "RESPonse FAST" );
	GpStatus = gp_wrt( GpAdrsBuf, "*OPC?" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return -1;
	}

	memset(RcvBuf, 0x00, sizeof(RcvBuf));
#ifdef _USE_U2
	GpStatus = U2Gpred(fd, GpAdrsBuf, RcvBuf, sizeof(RcvBuf));
#else
	GpStatus = gp_red( GpAdrsBuf, RcvBuf, sizeof(RcvBuf) );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_red() error [%d]\n", GpStatus);
		return -1;
	}

	if(RcvBuf[0] == '1'){
		;
	}else{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "動作完了していません。");
	}

	InitFlag = 1;
	return 0;
}

static int MyGpMeasure( void )
{
	int		GpStatus;
	char	RcvBuf[256];
	char	VolBuf[16];
	char	CurBuf[16];
	char	PowBuf[16];
	char	tmpBuf[8];
	double	datVal;
	long	unit;
	double	VolVal;
	long	Volunit;
	double	CurVal;
	long	Curunit;
	double	PowVal;
	char	unitBuf[8]; // m, u, n , k, M ,G ,,,,,
	unsigned char	bcnt;

	// 測定値を取得	
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "MEASure? V,A,W");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "MEASure? V,A,W" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return -1;
	}

	// 測定値をリード	
	memset(RcvBuf, 0x00, sizeof(RcvBuf));
#ifdef _USE_U2
	GpStatus = U2Gpred(fd, GpAdrsBuf, RcvBuf, sizeof(RcvBuf));
#else
	GpStatus = gp_red( GpAdrsBuf, RcvBuf, sizeof(RcvBuf) );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_red() error [%d]\n", GpStatus);
		return -1;
	}

	// 電圧・電流・電力 毎に振り分け
	// データフォーマット:V(ヘッダ部) ±□□□□□□E±□ (12Byte)
	memset(VolBuf, 0x00, sizeof(VolBuf));
	memset(CurBuf, 0x00, sizeof(CurBuf));
	memset(PowBuf, 0x00, sizeof(PowBuf));
	memcpy(VolBuf, RcvBuf+2, 10);
	memcpy(CurBuf, RcvBuf+15, 10);
	memcpy(PowBuf, RcvBuf+28, 10);

	for(bcnt = 0; bcnt < 3; bcnt++)
	{
		if(bcnt == 0)
		{
			// 取得した文字列からデータ抽出
			// データフォーマット: ±□□□□□□E±□ (10Byte)
			memset(tmpBuf, 0x00, sizeof(tmpBuf));
			memcpy(tmpBuf, VolBuf, 7); // 仮数部
			VolVal = atof(tmpBuf);
			datVal = VolVal;

			memset(tmpBuf, 0x00, sizeof(tmpBuf));
			memcpy(tmpBuf, VolBuf+8, 2); // 指数部
			unit = atoi(tmpBuf);
			Volunit = unit;
		}
		else if(bcnt == 1)
		{
			// 取得した文字列からデータ抽出
			// データフォーマット: ±□□□□□□E±□ (10Byte)
			memset(tmpBuf, 0x00, sizeof(tmpBuf));
			memcpy(tmpBuf, CurBuf, 7); // 仮数部
			CurVal = atof(tmpBuf);
			datVal = CurVal;

			memset(tmpBuf, 0x00, sizeof(tmpBuf));
			memcpy(tmpBuf, CurBuf+8, 2); // 指数部
			unit = atoi(tmpBuf);
			Curunit = unit;

			if(g_setpos >= X_RANGE)
			{
				// 次のデータセットポイントを0に戻す(リングバッファ)
				g_setpos = 0;
			}

			if((CurVal == 999.99 && unit == 9)||(CurVal == 888.88 && unit == 9)||(CurVal == 777.77 && unit == 9))
			{
				// データ取得できない場合は、前のデータ(または0)をセットしておく
				if(g_setpos != 0)
					g_Val[0][g_setpos] = g_Val[0][g_setpos - 1];
				else
					g_Val[0][g_setpos] = 0.0;
			}
			else
			{
				// データ取得できた場合は、グラフ表示のためのデータセット
				g_Val[0][g_setpos] = CurVal * pow(10.0, unit);
			}
		
		}
		else
		{
			// 取得した文字列からデータ抽出
			// データフォーマット: ±□□□□□□E±□ (10Byte)
			memset(tmpBuf, 0x00, sizeof(tmpBuf));
			memcpy(tmpBuf, PowBuf, 7); // 仮数部
			PowVal = atof(tmpBuf);
			datVal = PowVal;

			memset(tmpBuf, 0x00, sizeof(tmpBuf));
			memcpy(tmpBuf, PowBuf+8, 2); // 指数部
			unit = atoi(tmpBuf);

			if(g_setpos >= X_RANGE)
			{
				// 次のデータセットポイントを0に戻す(リングバッファ)
				g_setpos = 0;
			}

			if((PowVal == 999.99 && unit == 9)||(PowVal == 888.88 && unit == 9)||(PowVal == 777.77 && unit == 9))
			{
				// データ取得できない場合は、前のデータ(または0)をセットしておく
				if(g_setpos != 0)
					g_Val[1][g_setpos] = g_Val[1][g_setpos - 1];
				else
					g_Val[1][g_setpos] = 0.0;
			}
			else
			{
				// データ取得できた場合は、グラフ表示のためのデータセット
				g_Val[1][g_setpos] = PowVal * pow(10.0, unit);
			}
			g_getcount++;
			g_setpos++;


		}

		// 指数部を解析し、単位を変更
		memset(unitBuf, 0x00, sizeof(unitBuf));
		if(unit < -12)
		{
			memcpy(unitBuf, "p", 1);
			// 指数部を値に組み込む
			datVal = datVal * pow(10.0, (unit+12));
		}
		else if(unit == -12)
		{
			memcpy(unitBuf, "p", 1);
		}
		else if(unit > -12 && unit < -9)
		{
			memcpy(unitBuf, "n", 1);
			// 指数部を値に組み込む
			datVal = datVal * pow(10.0, (unit+9));
		}
		else if(unit == -9)
		{
			memcpy(unitBuf, "n", 1);
		}
		else if(unit > -9 && unit < -6)
		{
			memcpy(unitBuf, "u", 1);
			// 指数部を値に組み込む
			datVal = datVal * pow(10.0, (unit+6));
		}
		else if(unit == -6)
		{
			memcpy(unitBuf, "u", 1);
		}
		else if(unit > -6 && unit < -3)
		{
			memcpy(unitBuf, "m", 1);
			// 指数部を値に組み込む
			datVal = datVal * pow(10.0, (unit+3));
		}
		else if(unit == -3)
		{
			memcpy(unitBuf, "m", 1);
		}
		else if(unit > -3 && unit < 3)
		{
			// 指数部を値に組み込む
			datVal = datVal * pow(10.0, unit);
		}
		else if(unit == 3)
		{
			memcpy(unitBuf, "k", 1);
		}
		else if(unit > 3 && unit < 6)
		{
			memcpy(unitBuf, "k", 1);
			// 指数部を値に組み込む
			datVal = datVal * pow(10.0, (unit-3));
		}
		else if(unit == 6)
		{
			memcpy(unitBuf, "M", 1);
		}
		else if(unit > 6 && unit < 9)
		{
			memcpy(unitBuf, "M", 1);
			// 指数部を値に組み込む
			datVal = datVal * pow(10.0, (unit-6));
		}
		else if(unit == 9)
		{
			memcpy(unitBuf, "G", 1);
		}
		else if(unit > 9 && unit < 12)
		{
			memcpy(unitBuf, "G", 1);
			// 指数部を値に組み込む
			datVal = datVal * pow(10.0, (unit-9));
		}
		else if(unit == 12)
		{
			memcpy(unitBuf, "T", 1);
		}
		else if(unit > 12)
		{
			memcpy(unitBuf, "T", 1);
			// 指数部を値に組み込む
			datVal = datVal * pow(10.0, (unit-12));
		}

		if(bcnt == 0)
		{
			// 表示
			if((VolVal == 999.99 && unit == 9)||(VolVal == 888.88 && unit == 9)||(VolVal == 777.77 && unit == 9))
				sprintf(szBuf, "電圧: ------");
			else
				sprintf(szBuf, "電圧: %-+10g [%sV]", datVal, unitBuf);
			gtk_statusbar_push (GTK_STATUSBAR (vol_bar), GPOINTER_TO_INT (vol_id), szBuf);
		}
		else if(bcnt == 1)
		{
			// 表示
			if((CurVal == 999.99 && unit == 9)||(CurVal == 888.88 && unit == 9)||(CurVal == 777.77 && unit == 9))
				sprintf(szBuf, "電流: ------");
			else
				sprintf(szBuf, "電流: %-+10g [%sA]", datVal, unitBuf);
			gtk_statusbar_push (GTK_STATUSBAR (cur_bar), GPOINTER_TO_INT (cur_id), szBuf);
		}
		else
		{
			// 表示
			if((PowVal == 999.99 && unit == 9)||(PowVal == 888.88 && unit == 9)||(PowVal == 777.77 && unit == 9))
				sprintf(szBuf, "電力: ------");
			else
				sprintf(szBuf, "電力: %-+10g [%sW]", datVal, unitBuf);
			gtk_statusbar_push (GTK_STATUSBAR (pow_bar), GPOINTER_TO_INT (pow_id), szBuf);
		}

	} // End of for(bcnt = 0; bcnt < 3; bcnt++)

	sprintf(szBuf, "%ld 秒", g_getcount);
	gtk_statusbar_push (GTK_STATUSBAR (count_bar), GPOINTER_TO_INT (count_id), szBuf);

	return 0;
}

static gboolean timeout_callback( gpointer data )
{
	if( MyGpMeasure() != 0 )
	{
		//gtk_timeout_remove (timer);
		timer = 0;
		RunFlag = 0;
		return FALSE;
	}

	//PlotDataWave( hwnd, PlotVal );

	/* これがタイムアウト関数なので、
	* この関数が連続して呼び出されるように TRUE を返します。*/
	return TRUE;
}

static void Cmd_OnCmdGpInit( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;

	g_getcount = 0;

	if( RunFlag == 1 )
	{
		if(timer){
			gtk_timeout_remove (timer);
			timer = 0;
		}
		RunFlag = 0;
	}

	GpStatus = InitGpMachine();
	if( GpStatus != 0 )
	{
		sprintf(szBuf, "Initialize error: %d", GpStatus);
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), szBuf);
	}else{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "初期化終了");
	}
}

static void Cmd_OnCmdOutput( GtkWidget *widget,
                       gpointer   data )
{
	if(InitFlag == 0){
		return;
	}
	if( RunFlag == 1 )
	{
		if(timer){
			gtk_timeout_remove (timer);
			timer = 0;
		}
		RunFlag = 0;
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "測定終了");
		return;
	}

	if( MyGpMeasure() != 0 )
	{
		printf("Cmd_OnCmdOutput:MyGpMeasure() error \n");
		return;
	}
	gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "測定開始");

	timer = gtk_timeout_add (1000, timeout_callback, data);

	RunFlag = 1;
}

static void Cmd_OnCancel( GtkWidget *widget,
                      gpointer   data )
{
	if( RunFlag == 1 ){
		if(timer){
			gtk_timeout_remove (timer);
			timer = 0;
		}
		RunFlag = 0;
	}

	if(InitFlag){
		//send LCL command
#ifdef _USE_U2
		U2Gplcl(fd, GpAdrsBuf);
#else
		gp_lcl( GpAdrsBuf );
#endif
	}
}

static void enter_adrs( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
  sprintf(GpAdrsBuf, "%s", entry_text);
  //printf ("GpMachineAdrs: %d\n", GpMachineAdrs);
}

static void create_window( void )
{
	GtkWidget *window;
	GtkWidget *table;
	GtkWidget *table1;
	GtkWidget *frame;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *temp_bar;
	GtkWidget *area;
	GtkWidget *vrule;
	GtkWidget *button;
	gint temp_id;
	char	buf[12];

    /* make new window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);
    gtk_widget_set_size_request (GTK_WIDGET (window), 500, 400);
    gtk_window_set_title (GTK_WINDOW (window), "日置電気 3332");
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_main_quit), NULL);

    /* 縦横 13x13 のテーブルを生成します。*/
    table = gtk_table_new (13, 13, TRUE);
    gtk_container_add (GTK_CONTAINER (window), table);
    gtk_widget_show (table);

    /* make GPIB adrs */
    /* 新しいフレームを作成します。*/
    frame = gtk_frame_new ("GPIB 機器アドレス設定");
    gtk_table_attach_defaults (GTK_TABLE (table), frame, 0, 6, 0, 4);
    gtk_widget_show (frame);

    /* 縦横 5x4 のテーブルを生成します。*/
    table1 = gtk_table_new (5, 4, TRUE);
    gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
    gtk_table_set_row_spacing( GTK_TABLE (table1), 0, 5 );
    gtk_container_add (GTK_CONTAINER (frame), table1);
    gtk_widget_show (table1);

    label = gtk_label_new ("日置電気 3332 アドレス");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 0, 4, 0, 2);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 2);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_adrs),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "1");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_table_attach_defaults (GTK_TABLE (table1), entry, 4, 5, 0, 2);
    gtk_widget_show (entry);

    label = gtk_label_new ("USB220 アドレス");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 0, 4, 2, 4);
    gtk_widget_show (label);

    temp_bar = gtk_statusbar_new ();
    temp_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (temp_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " %d", MyGpibAdrs);
    gtk_statusbar_push (GTK_STATUSBAR (temp_bar), temp_id, buf);
    gtk_table_attach_defaults (GTK_TABLE (table1), temp_bar, 4, 5, 2, 4);
    gtk_widget_show (temp_bar);

    vol_bar = gtk_statusbar_new ();
    vol_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (vol_bar), "Statusbar example");
    gtk_table_attach_defaults (GTK_TABLE (table), vol_bar, 7, 13, 0, 1);
    gtk_widget_show (vol_bar);

    cur_bar = gtk_statusbar_new ();
    cur_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (cur_bar), "Statusbar example");
    gtk_table_attach_defaults (GTK_TABLE (table), cur_bar, 7, 13, 1, 2);
    gtk_widget_show (cur_bar);

    pow_bar = gtk_statusbar_new ();
    pow_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (pow_bar), "Statusbar example");
    gtk_table_attach_defaults (GTK_TABLE (table), pow_bar, 7, 13, 2, 3);
    gtk_widget_show (pow_bar);

    count_bar = gtk_statusbar_new ();
    count_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (count_bar), "Statusbar example");
    gtk_table_attach_defaults (GTK_TABLE (table), count_bar, 11, 13, 3, 4);
    gtk_widget_show (count_bar);

#if 0
    /* make graph */
    label = gtk_label_new ("[Volt]");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 4, 6, 0, 1);
    gtk_widget_show (label);

    label = gtk_label_new ("計測データ波形");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 6, 10, 0, 1);
    gtk_widget_show (label);

    /* ルーラを配置するテーブルと、ドローイング・エリアを作成します。*/
    table1 = gtk_table_new (3, 2, FALSE);
    gtk_table_attach_defaults (GTK_TABLE (table), table1, 5, 12, 1, 10);
    gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
    //gtk_table_set_col_spacing( GTK_TABLE (table1), 1, 2 );
    gtk_widget_show (table1);

    area = gtk_drawing_area_new ();
    //gtk_widget_set_size_request (GTK_WIDGET (area), 300, 200);
    gtk_drawing_area_size (GTK_DRAWING_AREA (area), 300, 200);
    gtk_table_attach (GTK_TABLE (table1), area, 1, 2, 1, 2,
                      GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
    //gtk_widget_set_events (area, GDK_POINTER_MOTION_MASK |
    //                             GDK_POINTER_MOTION_HINT_MASK);
    gtk_widget_show (area);

    //gdkgc = gdk_gc_new(GDK_WINDOW (area));

    vrule = gtk_vruler_new ();
    gtk_ruler_set_metric (GTK_RULER (vrule), GTK_PIXELS);
    gtk_ruler_set_range (GTK_RULER (vrule), 10, -10, 0, 200 );
    //g_signal_connect_swapped (G_OBJECT (area), "motion_notify_event",
    //                          G_CALLBACK (EVENT_METHOD (vrule, motion_notify_event)),
    //                          G_OBJECT (vrule));
    gtk_table_attach (GTK_TABLE (table1), vrule, 0, 1, 1, 2,
                      GTK_FILL, GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0);
    gtk_widget_show (vrule);
#endif

    status_bar = gtk_statusbar_new ();
    status_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (status_bar), "Statusbar example");
    gtk_table_attach_defaults (GTK_TABLE (table), status_bar, 1, 12, 11, 12);
    gtk_widget_show (status_bar);

    /* make botton */
    hbox = gtk_hbox_new (FALSE, 2);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_table_attach_defaults (GTK_TABLE (table), hbox, 3, 12, 12, 13);
    gtk_widget_show (hbox);

	button = gtk_button_new_with_label ("初期化");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdGpInit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);              

	button = gtk_button_new_with_label ("計測開始");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdOutput), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);

	button = gtk_button_new_with_label ("終了");
	g_signal_connect (G_OBJECT (button), "clicked",
		               G_CALLBACK (Cmd_OnCancel), NULL);
	g_signal_connect (G_OBJECT (button), "clicked",
			      G_CALLBACK (gtk_main_quit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (button);
	gtk_widget_show (button);

	gtk_widget_show (window);
}

int main( int   argc,
          char *argv[] )
{
#ifdef _USE_U2
	int id, i;
	unsigned short UnitId[MAX_USB_DEVICES];
	unsigned short MaxUnit = MAX_USB_DEVICES;

	// 引数無しの場合
	if (argc != 2) {
		MaxUnit = U2GpEnumUnit(UnitId, MaxUnit);
		for(i=0;i<MaxUnit;i++){
			printf("U2GpEnumUnit%d UnitId = %02x \n", i, UnitId[i]);
		}
		//最初に見つけたUSB220をオープン
		fd=U2GpOpenUnit(UnitId[0]);
	}else{
		id = atoi (argv[1]);
		//入力されたIDをもつUSB220をオープン
		fd=U2GpOpenUnit(id);
	}
#else
	fd=gp_attach();
#endif
	if(fd<=0){
		printf("Device not opened \n");
		return 0;
    }
	InitFlag = 0;
	gp_GetDeviceID(fd, 0, (char*)&MyGpibAdrs);

	gtk_init (&argc, &argv);

	create_window();

	gtk_main ();
	gp_detach(fd);

	return 0;
}
