//gcc -g r6552.c -o r6552 -L/usr/local/lib/ -lusb220_gpib -lpthread \
//	`pkg-config gtk+-2.0 --cflags --libs`
// caution: You need ROOT previledge when execute this utility
#include <stdlib.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "usb220_gpib.h"

#define _USE_U2
#define MAX_USB_DEVICES				16


int fd=0;
short MyGpibAdrs = 1;	// GPIB address for USB-GPIB
char GpAdrsBuf[12];	// GPIB address for target-GPIB
int InitFlag;
GtkWidget *status_bar;
gint context_id;



static void Cmd_OnCmdGpInit( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;
	char	szCommand[] = "H0KM01";

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return;
	}

	// enable IFC
#ifdef _USE_U2
	GpStatus = U2Gpcli(fd);
#else
	GpStatus = gp_cli();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_cli() error [%d]\n", GpStatus);
		return;
	}

	// enable REN
#ifdef _USE_U2
	GpStatus = U2Gpren(fd);
#else
	GpStatus = gp_ren();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_ren() error [%d]\n", GpStatus);
		return;
	}

	// Send SDC command
	// get address of GPIBdevice for R6552
#ifdef _USE_U2
	GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
	GpStatus = gp_clr( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_clr() error [%d]\n", GpStatus);
		return;
	}

	InitFlag = 1;
}

static void Cmd_OnCmdStart( GtkWidget *widget,
                      gpointer   data )
{
	int 	GpStatus;
	char  	RcvData[256];
	unsigned char	StatusByte[16];
	//int length;
	//char *pbuf;
	//int i;
	gchar *buff;

	if(InitFlag == 0){
		return;
	}

	// Send R6552 GPIB command
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "F3,R0,M1,PR1,H1,DL0");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "F3,R0,M1,PR1,H1,DL0" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	// do triger command
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return;
	}

	// read data from GPIB bus
	memset( RcvData, 0x00, sizeof(RcvData) );
#ifdef _USE_U2
	GpStatus = U2Gpred(fd, GpAdrsBuf, RcvData, sizeof(RcvData));
#else
	GpStatus = gp_red( GpAdrsBuf, RcvData, sizeof(RcvData) );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_red() error [%d]\n", GpStatus);
		return;
	}

#if 1
	RcvData[13] = 0x00;
	buff = g_strdup_printf ("%s", RcvData);
	gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, buff);
	g_free (buff);
#else
	length = gp_count();
	for(i=0;i<length;i++){
		if(i%16==0){
			printf("\n");
		}
		printf("%02x ", RcvData[i]);
	}
	printf("\nlength = %d\n", length);
	pbuf = (char *)malloc(length);
	memcpy(pbuf, RcvData, length);
	buff = g_strdup_printf ("%s", pbuf);
	gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, buff);
	g_free (buff);
	free(pbuf);
#endif
	//printf ("read data OK: StatusByte = 0x%02x\n", StatusByte[0]);
}

static void Cmd_OnCancel( GtkWidget *widget,
                      gpointer   data )
{
	int	GpStatus;

	if(InitFlag == 0){
		return;
	}

	//send LCL command
#ifdef _USE_U2
	GpStatus = U2Gplcl(fd, GpAdrsBuf);
#else
	GpStatus = gp_lcl( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_lcl() error [%d]\n", GpStatus);
	}
}

static void enter_callback( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  //printf ("changed contents: %s\n", entry_text);
  memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
  sprintf(GpAdrsBuf, "%s", entry_text);
}

static void create_window( void )
{
	GtkWidget *window;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *frame;
	GtkWidget *button;
	char	buf[12];

    /* make new window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);
    gtk_widget_set_size_request (GTK_WIDGET (window), 340, 180);
    gtk_window_set_title (GTK_WINDOW (window), "R6552");
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_main_quit), NULL);

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 5);
    gtk_widget_show (hbox);

    label = gtk_label_new ("GPIB 機器アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 1);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_callback),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "1");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
    gtk_widget_show (entry);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 5);
    gtk_widget_show (hbox);

    label = gtk_label_new ("USB220 アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 10);
    gtk_widget_show (label);

    status_bar = gtk_statusbar_new ();
    context_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (status_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " %d", MyGpibAdrs);
    gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, buf);
    gtk_box_pack_start (GTK_BOX (hbox), status_bar, TRUE, TRUE, 0);
    gtk_widget_show (status_bar);

    /* 新しいフレームを作成します。*/
    frame = gtk_frame_new ("R6552");
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 5);
    gtk_widget_show (frame);

    hbox = gtk_hbox_new (FALSE, 5);
    gtk_container_add (GTK_CONTAINER (frame), hbox);
    gtk_widget_show (hbox);

    label = gtk_label_new ("測定値");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    status_bar = gtk_statusbar_new ();
    context_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (status_bar), "Statusbar example");
    gtk_box_pack_start (GTK_BOX (hbox), status_bar, TRUE, TRUE, 5);
    gtk_widget_show (status_bar);

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 10);
    gtk_widget_show (hbox);

	button = gtk_button_new_with_label ("初期化");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdGpInit), GINT_TO_POINTER (context_id));
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);              

	button = gtk_button_new_with_label ("計測");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdStart), GINT_TO_POINTER (context_id));
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);

	button = gtk_button_new_with_label ("終了");
	g_signal_connect (G_OBJECT (button), "clicked",
		               G_CALLBACK (Cmd_OnCancel), GINT_TO_POINTER (context_id));
	g_signal_connect (G_OBJECT (button), "clicked",
			      G_CALLBACK (gtk_main_quit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (button);
	gtk_widget_show (button);

	gtk_widget_show (window);
}

int main( int   argc,
          char *argv[] )
{
#ifdef _USE_U2
	int id, i;
	unsigned short UnitId[MAX_USB_DEVICES];
	unsigned short MaxUnit = MAX_USB_DEVICES;

	// 引数無しの場合
	if (argc != 2) {
		MaxUnit = U2GpEnumUnit(UnitId, MaxUnit);
		for(i=0;i<MaxUnit;i++){
			printf("U2GpEnumUnit%d UnitId = %02x \n", i, UnitId[i]);
		}
		//最初に見つけたUSB220をオープン
		fd=U2GpOpenUnit(UnitId[0]);
	}else{
		id = atoi (argv[1]);
		//入力されたIDをもつUSB220をオープン
		fd=U2GpOpenUnit(id);
	}
#else
	fd=gp_attach();
#endif
	if(fd<=0){
		printf("Device not opened \n");
		return 0;
    }
	InitFlag = 0;
	gp_GetDeviceID(fd, 0, (char*)&MyGpibAdrs);

	gtk_init (&argc, &argv);

	create_window();

	gtk_main ();
	gp_detach(fd);

	return 0;
}
