/**********************************************************************************
 *
 *	電圧発生器YEW2553 制御プログラム
 *
 **********************************************************************************/
//gcc -g yew2553.c -o yew2553 -L/usr/local/lib/ -lusb220_gpib -lpthread \
//	`pkg-config gtk+-2.0 --cflags --libs`
// caution: You need ROOT previledge when execute this utility
#include <stdlib.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "usb220_gpib.h"

#define _USE_U2
#define MAX_USB_DEVICES				16


int fd=0;
short MyGpibAdrs = 1;	// GPIB address for USB-GPIB
char GpAdrsBuf[12];	// GPIB address for target-GPIB
char szBuf[256];
int InitFlag;
double Volt;


static void Cmd_OnCmdGpInit( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return;
	}

	// enable IFC
#ifdef _USE_U2
	GpStatus = U2Gpcli(fd);
#else
	GpStatus = gp_cli();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_cli() error [%d]\n", GpStatus);
		return;
	}

	// enable REN
#ifdef _USE_U2
	GpStatus = U2Gpren(fd);
#else
	GpStatus = gp_ren();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_ren() error [%d]\n", GpStatus);
		return;
	}

	// Send SDC command
	// get address of GPIBdevice
#ifdef _USE_U2
	GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
	GpStatus = gp_clr( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_clr() error [%d]\n", GpStatus);
		return;
	}

	InitFlag = 1;
}

static void Cmd_OnCmdStart( GtkWidget *widget,
                      gpointer   data )
{
	int 	GpStatus;
	char	InBuf[256];
	double	dtemp;
	int		itemp;

	if(InitFlag == 0){
		return;
	}

	if( Volt < 10 ){
		//>>> Voltを1000倍してintに変換
		dtemp = (double)(Volt * 1000);
		memset(szBuf, 0x00, sizeof(szBuf));
		sprintf(szBuf, "%6.1lf", dtemp);
		memset(&szBuf[4], 0x00, 2);
		itemp = atoi( szBuf );
		//printf ("itemp: %d\n", itemp);
		// Voltを1000倍してint型に変換 <<<
	}
	if( Volt == 0 )
		sprintf( InBuf, "S00000"  );
	else if( Volt < 0.01 && Volt >= 0.001 )
		sprintf( InBuf, "S0000%d", itemp );
	else if( Volt < 0.1 && Volt >= 0.01 )
		sprintf( InBuf, "S000%d", itemp );
	else if( Volt < 1 && Volt >= 0.1 )
		sprintf( InBuf, "S00%d", itemp );
	else if( Volt < 10 && Volt >= 1 )
		sprintf( InBuf, "S0%d", itemp );
	else if( Volt == 10 )
		sprintf( InBuf, "S10000" );
	else
	{
		printf("電圧値設定エラー\n");
		return;
	}

	// YEW2553電圧出力コマンドデータの設定
	// V3:10Vレンジ設定
	// P0:極性+
	// O0:出力オフ
	// Ex...10V電圧出力時のコマンド:"V3P0S0000000"
	memset( szBuf, 0x00, sizeof( szBuf ) );
	strcpy( szBuf, "V3P0");
	strcat( szBuf, InBuf );
	strcat( szBuf, "O0" );
	printf ("send command: %s\n", szBuf);

	// YEW2553の出力電圧設定
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	// do triger command
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return;
	}

	// 01:出力ON
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "01");
#else
	GpStatus = gp_wrt( GpAdrsBuf, szBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return;
	}
}

static void Cmd_OnCancel( GtkWidget *widget,
                      gpointer   data )
{
	if(InitFlag){
		// 0V、OFF状態に戻す
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "S00000");
		U2Gpwrt(fd, GpAdrsBuf, "00");
		U2Gptrg(fd, GpAdrsBuf);
#else
		gp_wrt( GpAdrsBuf, "S00000" );
		gp_wrt( GpAdrsBuf, "O0" );
		gp_trg( GpAdrsBuf );
#endif
	}
}

static void enter_adrs( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  //printf ("changed contents: %s\n", entry_text);
  memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
  sprintf(GpAdrsBuf, "%s", entry_text);
}

static void enter_volt( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;
  char	buf[12];

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  memset(buf, 0x00, sizeof(buf));
  sprintf(buf, "%s", entry_text);
  Volt = atof( buf );
  //printf ("volt contents: %06.4lf\n", Volt);
}

static void create_window( void )
{
	GtkWidget *window;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *frame;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *status_bar;
	GtkWidget *button;
	gint context_id;
	char	buf[12];

    /* make new window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);
    gtk_widget_set_size_request (GTK_WIDGET (window), 360, 180);
    gtk_window_set_title (GTK_WINDOW (window), "YEW2553");
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_main_quit), NULL);

    vbox = gtk_vbox_new (FALSE, 5);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);

    /* make GPIB adrs */
    hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    label = gtk_label_new ("GPIB 機器アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 1);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_adrs),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "4");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
    gtk_widget_show (entry);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    label = gtk_label_new ("USB220 アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    status_bar = gtk_statusbar_new ();
    context_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (status_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " %d", MyGpibAdrs);
    gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, buf);
    gtk_box_pack_start (GTK_BOX (hbox), status_bar, TRUE, TRUE, 0);
    gtk_widget_show (status_bar);

    /* 新しいフレームを作成します。*/
    frame = gtk_frame_new ("YEW2553 Maltimeter");
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
    gtk_widget_show (frame);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
    gtk_container_add (GTK_CONTAINER (frame), hbox);
    gtk_widget_show (hbox);

    label = gtk_label_new ("出力値(0〜10V)");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 6);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_volt),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " ");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
    gtk_widget_show (entry);

    /* make botton */
    hbox = gtk_hbox_new (FALSE, 2);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

	button = gtk_button_new_with_label ("初期化");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdGpInit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);              

	button = gtk_button_new_with_label ("出力");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdStart), GINT_TO_POINTER (context_id));
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);

	button = gtk_button_new_with_label ("終了");
	g_signal_connect (G_OBJECT (button), "clicked",
		               G_CALLBACK (Cmd_OnCancel), NULL);
	g_signal_connect (G_OBJECT (button), "clicked",
			      G_CALLBACK (gtk_main_quit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (button);
	gtk_widget_show (button);

	gtk_widget_show (window);
}

int main( int   argc,
          char *argv[] )
{
#ifdef _USE_U2
	int id, i;
	unsigned short UnitId[MAX_USB_DEVICES];
	unsigned short MaxUnit = MAX_USB_DEVICES;

	// 引数無しの場合
	if (argc != 2) {
		MaxUnit = U2GpEnumUnit(UnitId, MaxUnit);
		for(i=0;i<MaxUnit;i++){
			printf("U2GpEnumUnit%d UnitId = %02x \n", i, UnitId[i]);
		}
		//最初に見つけたUSB220をオープン
		fd=U2GpOpenUnit(UnitId[0]);
	}else{
		id = atoi (argv[1]);
		//入力されたIDをもつUSB220をオープン
		fd=U2GpOpenUnit(id);
	}
#else
	fd=gp_attach();
#endif
	if(fd<=0){
		printf("Device not opened \n");
		return 0;
    }
	InitFlag = 0;
	gp_GetDeviceID(fd, 0, (char*)&MyGpibAdrs);

	gtk_init (&argc, &argv);

	create_window();

	gtk_main ();
	gp_detach(fd);

	return 0;
}
